<?php

// phpcs:disable Drupal.NamingConventions

/**
 * Provides a Views field handler for selecting a weight in a specified range.
 */
class views_handler_weight_selector extends views_handler_field {

  /**
   * Initialize the field handler.
   *
   * @param object $view
   *   The view object.
   * @param array $options
   *   An array of options.
   */
  public function init(&$view, &$options) {
    parent::init($view, $options);

    $this->options['weight_selector'] = TRUE;
  }

  /**
   * Define options for the field handler.
   *
   * @return array
   *   An array of option definitions.
   */
  public function option_definition() {
    $options = parent::option_definition();

    $options['weight_range'] = [
      'default' => 20,
    ];

    return $options;
  }

  /**
   * Build the options form for the field handler.
   *
   * @param array $form
   *   An array representing the form.
   * @param array $form_state
   *   An array representing the form state.
   */
  public function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $form['weight_range'] = [
      '#type' => 'textfield',
      '#title' => t('Range'),
      '#description' => t('The range of weights available to select. For example, a range of 20 will allow you to select a weight between -20 and 20.'),
      '#default_value' => $this->options['weight_range'],
      '#size' => 5,
    ];
  }

  /**
   * Render the weight selector.
   *
   * @param array $values
   *   An array of values.
   *
   * @return string
   *   The rendered weight selector HTML.
   */
  public function render($values) {
    return '<!--form-item-' . $this->options['id'] . '--' . $this->view->row_index . '-->';
  }

  /**
   * Check access for the weight selector.
   *
   * @return bool
   *   TRUE if the user has access to assign node weights; otherwise, FALSE.
   */
  public function access() {
    return user_access('assign node weight');
  }

  /**
   * Build the form for the weight selector.
   *
   * @param array $form
   *   An array representing the form.
   * @param array $form_state
   *   An array representing the form state.
   */
  public function views_form(&$form, &$form_state) {
    // The view is empty, abort.
    if (empty($this->view->result)) {
      $form['#access'] = FALSE;
      return;
    }

    $form[$this->options['id']] = [
      '#tree' => TRUE,
    ];

    $options = _weight_get_options($this->options['weight_range']);
    $entities = entity_get_info();

    foreach ($entities as $type => $entity_info) {
      if (isset($entity_info['base table']) && $this->view->base_table == $entity_info['base table']) {
        $entity_type = $type;
        $entity_key = $entity_info['entity keys']['id'];

        // 2096231: patch for draggable views.
        $field = $this->view->field[$entity_key];
        if (!empty($field->field_alias)) {
          $entity_key = $field->field_alias;
        }
      }
    }

    // At this point, query has already been run, so we can access the results.
    foreach ($this->view->result as $row_index => $row) {
      $entity = entity_load($entity_type, [$row->{$entity_key}]);

      $form[$this->options['id']][$row_index]['weight'] = [
        '#type' => 'select',
        '#title' => t('Weight for row @number', ['@number' => $row_index + 1]),
        '#title_display' => 'invisible',
        '#options' => $options,
        '#default_value' => $this->get_value($row),
        '#attributes' => ['class' => ['weight-selector']],
      ];

      $form[$this->options['id']][$row_index]['entity'] = [
        '#type' => 'value',
        '#value' => $entity,
      ];
    }

    $form['entity_type'] = [
      '#type' => 'value',
      '#value' => $entity_type,
    ];

    $form['views_field'] = [
      '#type' => 'value',
      '#value' => $this->field,
    ];

    $form['#action'] = request_uri();
  }

  /**
   * Submit handler for the Views form.
   *
   * @param array $form
   *   An array representing the form.
   * @param array $form_state
   *   An array representing the form state.
   */
  public function views_form_submit($form, &$form_state) {
    $values = $form_state['values'];
    $field_name = str_replace('_selector', '', $values['views_field']);

    foreach ($values[$values['views_field']] as $value) {
      $entity = array_pop($value['entity']);
      $lang = field_language($values['entity_type'], $entity, $field_name);
      $entity->{$field_name}[$lang][0]['value'] = $value['weight'];
      entity_save($values['entity_type'], $entity);
    }
  }

}

// phpcs:enable
